@@ -1,32 +1,32 @@
 /* -*- mode: js2; js2-basic-offset: 4; indent-tabs-mode: nil -*- */
 
-const Big = imports.gi.Big;
 const Clutter = imports.gi.Clutter;
 const Gio = imports.gi.Gio;
 const GLib = imports.gi.GLib;
 const Lang = imports.lang;
-const Mainloop = imports.mainloop;
 const Meta = imports.gi.Meta;
+const St = imports.gi.St;
 const Shell = imports.gi.Shell;
 const Signals = imports.signals;
 const Gettext = imports.gettext.domain('gnome-shell');
 const _ = Gettext.gettext;
 
-const Lightbox = imports.ui.lightbox;
+const FileUtils = imports.misc.fileUtils;
 const Main = imports.ui.main;
+const ModalDialog = imports.ui.modalDialog;
+const Tweener = imports.ui.tweener;
+const Util = imports.misc.util;
+const History = imports.misc.history;
 
-const BOX_BACKGROUND_COLOR = new Clutter.Color();
-BOX_BACKGROUND_COLOR.from_pixel(0x000000cc);
-
-const BOX_TEXT_COLOR = new Clutter.Color();
-BOX_TEXT_COLOR.from_pixel(0xffffffff);
-
-const DIALOG_WIDTH = 320;
-const DIALOG_PADDING = 6;
-const ICON_SIZE = 24;
-const ICON_BOX_SIZE = 36;
 const MAX_FILE_DELETED_BEFORE_INVALID = 10;
 
+const HISTORY_KEY = 'command-history';
+
+const LOCKDOWN_SCHEMA = 'org.gnome.desktop.lockdown';
+const DISABLE_COMMAND_LINE_KEY = 'disable-command-line';
+
+const DIALOG_GROW_TIME = 0.1;
+
 function CommandCompleter() {
     this._init();
 }
@@ -35,6 +35,7 @@ CommandCompleter.prototype = {
     _init : function() {
         this._changedCount = 0;
         this._paths = GLib.getenv('PATH').split(':');
+        this._paths.push(GLib.get_home_dir());
         this._valid = false;
         this._updateInProgress = false;
         this._childs = new Array(this._paths.length);
@@ -57,7 +58,7 @@ CommandCompleter.prototype = {
             this._paths[i] = file.get_path();
             this._monitors[i] = file.monitor_directory(Gio.FileMonitorFlags.NONE, null);
             if (this._monitors[i] != null) {
-                this._monitors[i].connect("changed", Lang.bind(this, this._onChanged));
+                this._monitors[i].connect('changed', Lang.bind(this, this._onChanged));
             }
         }
         this._paths = this._paths.filter(function(a) {
@@ -66,25 +67,6 @@ CommandCompleter.prototype = {
         this._update(0);
     },
 
-    _onGetEnumerateComplete : function(obj, res) {
-        this._enumerator = obj.enumerate_children_finish(res);
-        this._enumerator.next_files_async(100, GLib.PRIORITY_LOW, null, Lang.bind(this, this._onNextFileComplete), null);
-    },
-
-    _onNextFileComplete : function(obj, res) {
-        let files = obj.next_files_finish(res);
-        for (let i = 0; i < files.length; i++) {
-            this._childs[this._i].push(files[i].get_name());
-        }
-        if (files.length) {
-            this._enumerator.next_files_async(100, GLib.PRIORITY_LOW, null, Lang.bind(this, this._onNextFileComplete), null);
-        } else {
-            this._enumerator.close(null);
-            this._enumerator = null;
-            this._update(this._i + 1);
-        }
-    },
-
     update : function() {
         if (this._valid)
             return;
@@ -104,7 +86,12 @@ CommandCompleter.prototype = {
         }
         let file = Gio.file_new_for_path(this._paths[i]);
         this._childs[this._i] = [];
-        file.enumerate_children_async(Gio.FILE_ATTRIBUTE_STANDARD_NAME, Gio.FileQueryInfoFlags.NONE, GLib.PRIORITY_LOW, null, Lang.bind(this, this._onGetEnumerateComplete), null);
+        FileUtils.listDirAsync(file, Lang.bind(this, function (files) {
+            for (let i = 0; i < files.length; i++) {
+                this._childs[this._i].push(files[i].get_name());
+            }
+            this._update(this._i + 1);
+        }));
     },
 
     _onChanged : function(m, f, of, type) {
@@ -138,7 +125,7 @@ CommandCompleter.prototype = {
     },
 
     getCompletion: function(text) {
-        let common = "";
+        let common = '';
         let notInit = true;
         if (!this._valid) {
             this._update(0);
@@ -151,7 +138,7 @@ CommandCompleter.prototype = {
                     break;
             }
             if (k == 0)
-                return "";
+                return '';
             return s1.substr(0, k);
         }
         function _hasPrefix(s1, prefix) {
@@ -176,17 +163,18 @@ CommandCompleter.prototype = {
 
 function RunDialog() {
     this._init();
-};
+}
 
 RunDialog.prototype = {
+__proto__: ModalDialog.ModalDialog.prototype,
     _init : function() {
-        this._isOpen = false;
+        ModalDialog.ModalDialog.prototype._init.call(this, { styleClass: 'run-dialog' });
 
-        let gconf = Shell.GConf.get_default();
-        gconf.connect('changed::development_tools', Lang.bind(this, function () {
-            this._enableInternalCommands = gconf.get_boolean('development_tools');
+        this._lockdownSettings = new Gio.Settings({ schema: LOCKDOWN_SCHEMA });
+        global.settings.connect('changed::development-tools', Lang.bind(this, function () {
+            this._enableInternalCommands = global.settings.get_boolean('development-tools');
         }));
-        this._enableInternalCommands = gconf.get_boolean('development_tools');
+        this._enableInternalCommands = global.settings.get_boolean('development-tools');
 
         this._internalCommands = { 'lg':
                                    Lang.bind(this, function() {
@@ -204,93 +192,67 @@ RunDialog.prototype = {
 
                                    'debugexit': Lang.bind(this, function() {
                                        Meta.exit(Meta.ExitCode.ERROR);
+                                   }),
+
+                                   // rt is short for "reload theme"
+                                   'rt': Lang.bind(this, function() {
+                                       Main.loadTheme();
                                    })
                                  };
 
-        // All actors are inside _group. We create it initially
-        // hidden then show it in show()
-        this._group = new Clutter.Group({ visible: false,
-                                          x: 0,
-                                          y: 0,
-                                          width: global.screen_width,
-                                          height: global.screen_height });
-        global.stage.add_actor(this._group);
 
-        let lightbox = new Lightbox.Lightbox(this._group, true);
+        let label = new St.Label({ style_class: 'run-dialog-label',
+                                   text: _("Please enter a command:") });
 
-        this._boxH = new Big.Box({ orientation: Big.BoxOrientation.HORIZONTAL,
-                                   x_align: Big.BoxAlignment.CENTER,
-                                   y_align: Big.BoxAlignment.CENTER });
+        this.contentLayout.add(label, { y_align: St.Align.START });
 
-        this._group.add_actor(this._boxH);
-        lightbox.highlight(this._boxH);
+        let entry = new St.Entry({ style_class: 'run-dialog-entry' });
 
-        let boxV = new Big.Box({ orientation: Big.BoxOrientation.VERTICAL,
-                                 y_align: Big.BoxAlignment.CENTER });
+        this._entryText = entry.clutter_text;
+        this.contentLayout.add(entry, { y_align: St.Align.START });
+        this.setInitialKeyFocus(this._entryText);
 
-        this._boxH.append(boxV, Big.BoxPackFlags.NONE);
+        this._errorBox = new St.BoxLayout({ style_class: 'run-dialog-error-box' });
 
+        this.contentLayout.add(this._errorBox, { expand: true });
 
-        let dialogBox = new Big.Box({ orientation: Big.BoxOrientation.VERTICAL,
-                                      background_color: BOX_BACKGROUND_COLOR,
-                                      corner_radius: 4,
-                                      reactive: false,
-                                      padding: DIALOG_PADDING,
-                                      width: DIALOG_WIDTH });
+        let errorIcon = new St.Icon({ icon_name: 'dialog-error', icon_size: 24, style_class: 'run-dialog-error-icon' });
 
-        this._boxH.append(dialogBox, Big.BoxPackFlags.NONE);
-
-        let label = new Clutter.Text({ color: BOX_TEXT_COLOR,
-                                       font_name: '18px Sans',
-                                       text: _("Please enter a command:") });
-
-        dialogBox.append(label, Big.BoxPackFlags.EXPAND);
-
-        this._entry = new Clutter.Text({ color: BOX_TEXT_COLOR,
-                                         font_name: '20px Sans Bold',
-                                         editable: true,
-                                         activatable: true,
-                                         singleLineMode: true });
-
-        dialogBox.append(this._entry, Big.BoxPackFlags.EXPAND);
-
-        this._errorBox = new Big.Box({ orientation: Big.BoxOrientation.HORIZONTAL,
-                                       padding_top: DIALOG_PADDING });
-
-        dialogBox.append(this._errorBox, Big.BoxPackFlags.EXPAND);
-
-        let iconBox = new Big.Box({ orientation: Big.BoxOrientation.VERTICAL,
-                                    y_align: Big.BoxAlignment.CENTER,
-                                    x_align: Big.BoxAlignment.CENTER,
-                                    width: ICON_BOX_SIZE,
-                                    height: ICON_BOX_SIZE });
-
-        this._errorBox.append(iconBox, Big.BoxPackFlags.NONE);
+        this._errorBox.add(errorIcon, { y_align: St.Align.MIDDLE });
 
         this._commandError = false;
 
-        let errorIcon = Shell.TextureCache.get_default().load_icon_name("gtk-dialog-error", ICON_SIZE);
-        iconBox.append(errorIcon, Big.BoxPackFlags.EXPAND);
+        this._errorMessage = new St.Label({ style_class: 'run-dialog-error-label' });
+        this._errorMessage.clutter_text.line_wrap = true;
 
-        this._errorMessage = new Clutter.Text({ color: BOX_TEXT_COLOR,
-                                                font_name: '18px Sans Bold',
-                                                line_wrap: true });
-
-        this._errorBox.append(this._errorMessage, Big.BoxPackFlags.EXPAND);
+        this._errorBox.add(this._errorMessage, { expand: true,
+                                                 y_align: St.Align.MIDDLE,
+                                                 y_fill: false });
 
         this._errorBox.hide();
 
-        this._entry.connect('activate', Lang.bind(this, function (o, e) {
-            this._run(o.get_text());
-            if (!this._commandError)
-                this.close();
-        }));
-
         this._pathCompleter = new Gio.FilenameCompleter();
         this._commandCompleter = new CommandCompleter();
         this._group.connect('notify::visible', Lang.bind(this._commandCompleter, this._commandCompleter.update));
-        this._entry.connect('key-press-event', Lang.bind(this, function(o, e) {
+
+        this._history = new History.HistoryManager({ gsettingsKey: HISTORY_KEY,
+                                                     entry: this._entryText });
+        this._entryText.connect('key-press-event', Lang.bind(this, function(o, e) {
             let symbol = e.get_key_symbol();
+            if (symbol == Clutter.Return || symbol == Clutter.KP_Enter) {
+                this.popModal();
+                if (Shell.get_event_state(e) & Clutter.ModifierType.CONTROL_MASK)
+                    this._run(o.get_text(), true);
+                else
+                    this._run(o.get_text(), false);
+                if (!this._commandError)
+                    this.close();
+                else {
+                    if (!this.pushModal())
+                        this.close();
+                }
+                return true;
+            }
             if (symbol == Clutter.Escape) {
                 this.close();
                 return true;
@@ -335,68 +297,74 @@ RunDialog.prototype = {
         }
     },
 
-    _run : function(command) {
+    _run : function(input, inTerminal) {
+        let command = input;
+
+        this._history.addItem(input);
         this._commandError = false;
         let f;
         if (this._enableInternalCommands)
-            f = this._internalCommands[command];
+            f = this._internalCommands[input];
         else
             f = null;
         if (f) {
             f();
-        } else if (command) {
+        } else if (input) {
             try {
-                let [ok, len, args] = GLib.shell_parse_argv(command);
-                let p = new Shell.Process({'args' : args});
-                p.run();
+                if (inTerminal)
+                    command = 'gnome-terminal -x ' + input;
+                Util.trySpawnCommandLine(command);
             } catch (e) {
-                this._commandError = true;
-                /*
-                 * The exception contains an error string like:
-                 * Error invoking Shell.run: Failed to execute child process "foo"
-                 * (No such file or directory)
-                 * We are only interested in the actual error, so parse that out.
-                 */
-                let m = /.+\((.+)\)/.exec(e);
-                let errorStr = _("Execution of '%s' failed:").format(command) + "\n" + m[1];
-                this._errorMessage.set_text(errorStr);
-                this._errorBox.show();
+                // Mmmh, that failed - see if @input matches an existing file
+                let path = null;
+                if (input.charAt(0) == '/') {
+                    path = input;
+                } else {
+                    if (input.charAt(0) == '~')
+                        input = input.slice(1);
+                    path = GLib.get_home_dir() + '/' + input;
+                }
+
+                if (GLib.file_test(path, GLib.FileTest.EXISTS)) {
+                    let file = Gio.file_new_for_path(path);
+                    Gio.app_info_launch_default_for_uri(file.get_uri(),
+                                                        global.create_app_launch_context());
+                } else {
+                    this._commandError = true;
+
+                    this._errorMessage.set_text(e.message);
+
+                    if (!this._errorBox.visible) {
+                        let [errorBoxMinHeight, errorBoxNaturalHeight] = this._errorBox.get_preferred_height(-1);
+
+                        let parentActor = this._errorBox.get_parent();
+                        Tweener.addTween(parentActor,
+                                         { height: parentActor.height + errorBoxNaturalHeight,
+                                           time: DIALOG_GROW_TIME,
+                                           transition: 'easeOutQuad',
+                                           onComplete: Lang.bind(this,
+                                               function() {
+                                                    parentActor.set_height(-1);
+                                                    this._errorBox.show();
+                                               })
+                                         });
+                    }
+                }
             }
         }
     },
 
-    open : function() {
-        if (this._isOpen) // Already shown
-            return;
-
-        if (!Main.pushModal(this._group))
-            return;
-
-        // Position the dialog on the current monitor
-        let monitor = global.get_focus_monitor();
-
-        this._boxH.set_position(monitor.x, monitor.y);
-        this._boxH.set_size(monitor.width, monitor.height);
-
-        this._isOpen = true;
-        this._group.show();
-
-        global.stage.set_key_focus(this._entry);
-    },
-
-    close : function() {
-        if (!this._isOpen)
-            return;
-
-        this._isOpen = false;
-        
+    open: function() {
+        this._history.lastItem();
         this._errorBox.hide();
+        this._entryText.set_text('');
         this._commandError = false;
 
-        this._group.hide();
-        this._entry.set_text('');
+        if (this._lockdownSettings.get_boolean(DISABLE_COMMAND_LINE_KEY))
+            return;
+
+        ModalDialog.ModalDialog.prototype.open.call(this);
+    },
 
-        Main.popModal(this._group);
-    }
 };
 Signals.addSignalMethods(RunDialog.prototype);
