@@ -93,7 +93,8 @@ Indicator.prototype = {
         this._updateFullMenu();
 
         this.menu.addAction(_("Bluetooth Settings"), function() {
-            GLib.spawn_command_line_async('gnome-control-center bluetooth');
+            let app = Shell.AppSystem.get_default().get_app('bluetooth-properties.desktop');
+            app.activate(-1);
         });
 
         this._applet.connect('pincode-request', Lang.bind(this, this._pinRequest));
@@ -136,6 +137,7 @@ Indicator.prototype = {
     _updateDevices: function() {
         let devices = this._applet.get_devices();
 
+        let newlist = [ ];
         for (let i = 0; i < this._deviceItems.length; i++) {
             let item = this._deviceItems[i];
             let destroy = true;
@@ -143,26 +145,20 @@ Indicator.prototype = {
                 // we need to deep compare because BluetoothSimpleDevice is a boxed type
                 // (but we take advantage of that, because _skip will disappear the next
                 // time get_devices() is called)
-                if (this._deviceCompare(item._device, devices[i])) {
-                    item.label.text = devices[i].alias;
-                    devices[i]._skip = true;
+                if (this._deviceCompare(item._device, devices[j])) {
+                    item.label.text = devices[j].alias;
+                    devices[j]._skip = true;
                     destroy = false;
+                    break;
                 }
             }
-            if (destroy) {
+            if (destroy)
                 item.destroy();
-                item._destroyed = true;
-            }
-        }
-
-        let newlist = [ ];
-        for (let i = 0; i < this._deviceItems.length; i++) {
-            let item = this._deviceItems[i];
-            if (!item._destroyed)
+            else
                 newlist.push(item);
         }
-        this._deviceItems = newlist;
 
+        this._deviceItems = newlist;
         this._hasDevices = newlist.length > 0;
         for (let i = 0; i < devices.length; i++) {
             let d = devices[i];
@@ -334,7 +330,7 @@ Source.prototype = {
     __proto__: MessageTray.Source.prototype,
 
     _init: function() {
-        MessageTray.Source.prototype._init.call(this, _("Bluetooth Agent"));
+        MessageTray.Source.prototype._init.call(this, _("Bluetooth"));
 
         this._setSummaryIcon(this.createNotificationIcon());
     },
@@ -368,7 +364,7 @@ AuthNotification.prototype = {
     _init: function(source, applet, device_path, name, long_name, uuid) {
         MessageTray.Notification.prototype._init.call(this,
                                                       source,
-                                                      _("Bluetooth Agent"),
+                                                      _("Bluetooth"),
                                                       _("Authorization request from %s").format(name),
                                                       { customContent: true });
         this.setResident(true);
@@ -408,7 +404,7 @@ ConfirmNotification.prototype = {
     _init: function(source, applet, device_path, name, long_name, pin) {
         MessageTray.Notification.prototype._init.call(this,
                                                       source,
-                                                      _("Bluetooth Agent"),
+                                                      _("Bluetooth"),
                                                       _("Pairing confirmation for %s").format(name),
                                                       { customContent: true });
         this.setResident(true);
@@ -441,7 +437,7 @@ PinNotification.prototype = {
     _init: function(source, applet, device_path, name, long_name, numeric) {
         MessageTray.Notification.prototype._init.call(this,
                                                       source,
-                                                      _("Bluetooth Agent"),
+                                                      _("Bluetooth"),
                                                       _("Pairing request for %s").format(name),
                                                       { customContent: true });
         this.setResident(true);
@@ -471,9 +467,15 @@ PinNotification.prototype = {
 
         this.connect('action-invoked', Lang.bind(this, function(self, action) {
             if (action == 'ok') {
-                if (this._numeric)
-                    this._applet.agent_reply_passkey(this._devicePath, parseInt(this._entry.text));
-                else
+                if (this._numeric) {
+                    let num = parseInt(this._entry.text);
+                    if (isNaN(num)) {
+                        // user reply was empty, or was invalid
+                        // cancel the operation
+                        num = -1;
+                    }
+                    this._applet.agent_reply_passkey(this._devicePath, num);
+                } else
                     this._applet.agent_reply_pincode(this._devicePath, this._entry.text);
             } else {
                 if (this._numeric)
